/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.mycompany._simulazione_museo;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.ArrayList;
import java.util.Scanner;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author gian
 */
public class UDPServer implements Runnable
{
    private DatagramSocket socket;
    //Massimo numero di byte dei dati di una request
    // e di una response
    private final int MAX_LENGHT_DATA=1024; 
   
    private String fileName="museo.csv";
    private ArrayList<Oggetto> elencoOggetti;
    
    public UDPServer(int port) throws SocketException
    {
        //Istanzia un socket per il server attivo sulla 
        //porta "port"
        socket=new DatagramSocket(port);
        //Imposta un Timeout, quando il server sarà in ascolto,
        //dopo un secondo si verifica un'eccezione SocketTimeoutException
        socket.setSoTimeout(1000);
        elencoOggetti=new ArrayList<Oggetto>();
        try 
        {
            caricaDatiDaFile();
        } 
        catch (IOException ex) 
        {
            //dati non caricati, gli oggetti non verranno trovati e risponderà ERROR
        }
    }
    
    public void run()
    {
        byte[] requestBuffer=new byte[MAX_LENGHT_DATA];
        DatagramPacket request=new DatagramPacket(requestBuffer, requestBuffer.length);
        String requestString;
        String responseString;
        DatagramPacket response;
        byte[] responseBuffer;
        //ciclo:
        //riceve request, costruisce repoinse, 
        //invia la response
        while(!Thread.interrupted())
        {
            try 
            {
                //metto il socket in ascolto sulla sua porta
                //dopo un secondo si verificherà l'eccezione
                socket.receive(request);
                //leggo i dati contenuti nella request (è una stringa)
                requestString=new String(requestBuffer, "UTF-8");
                //costruisco la risposta
               // responseString=requestString.substring(0,request.getLength()); //sotto forma di stringa
               requestString=requestString.substring(0,request.getLength());
               
               responseString=analisiRequest(requestString);
                
               responseBuffer=responseString.getBytes(); //sotto forma di array di byte
               response=new DatagramPacket(responseBuffer, responseString.length(), request.getAddress(), request.getPort());
                //spedire response
                socket.send(response);
            } 
            catch (SocketTimeoutException ex) 
            {
                System.out.println("TIMEOUT");
            }
            catch (IOException ex) 
            {
                System.out.println("Errore di comunicazione");
            }
        }
        
        socket.close(); 
    }
    
    
    public static void main(String[] args) 
    {
         Scanner tastiera=new Scanner(System.in);
        try 
        {
            UDPServer UDPserver=new UDPServer(7);
            Thread server=new Thread(UDPserver);
            server.start();
            tastiera.nextLine();
            server.interrupt();
            server.join();
            System.out.println("Server interrotto");
        } 
        catch (SocketException ex) 
        {
            System.out.println("Impossibile istanziare il socket del server");
        } 
        catch (InterruptedException ex) 
        {
             System.out.println("Impossibile interrompere il server");
        }
    }

    private void caricaDatiDaFile() throws IOException
    {
        
        TextFile file=new TextFile(fileName, 'R');
        String rigaLetta;
        String[] elementiRiga;
        Oggetto o;
   
        
        try 
        {
            while(true)
            {
                rigaLetta=file.fromFile();
                elementiRiga=rigaLetta.split(";");
                o=new Oggetto(Integer.parseInt(elementiRiga[0]), elementiRiga[1], Integer.parseInt(elementiRiga[2]));
                elencoOggetti.add(o);
            }

        } 
        catch (FileException ex) 
        {
            //dati caricaricati correttamente
        }
        file.close();
    }
    
    private void scriviSuFile() throws IOException, FileException
    {
        TextFile file=new TextFile(fileName, 'W');
        for (Oggetto o:elencoOggetti)
        {
            file.toFile(o.toString());
        }
        file.close();
    }

    private String analisiRequest(String requestString) 
    {
       String[] elementiRequest=requestString.split(";");
       int codiceOggetto;
      
       try
       {
            if( elementiRequest.length==1)    
            {
                //richiesta informazioni oggetto e incremento numeri di letture
                codiceOggetto=Integer.parseInt(elementiRequest[0]);
                for(Oggetto o:elencoOggetti)
                {
                    if (o.getCodice()==codiceOggetto) //oggetto trovato
                    {
                        int contaRichieste=o.getNumeroRichieste();
                        contaRichieste++;
                        o.setNumeroRichieste(contaRichieste);
                        try 
                        {
                            scriviSuFile();  //aggiorno oggetto sul file
                        } 
                        catch (IOException | FileException ex) 
                        {
                            return "ERROR";
                        } 
                        return o.getDescrizione();
                    }
                }  
                return "ERROR";
            }
            else if (elementiRequest.length==2)
            {
                //richiesta numero di letture
               if (!elementiRequest[0].equals("?"))
                   return "ERROR";
               codiceOggetto=Integer.parseInt(elementiRequest[1]);
               for(Oggetto o:elencoOggetti)
                {
                    if (o.getCodice()==codiceOggetto) //oggetto trovato
                    {
                        int numeroRichieste=o.getNumeroRichieste();
                        return Integer.toString(o.getNumeroRichieste());
                    }
                }
               return "ERROR"; //oggetto non trovato
            }
            else
            {
                return "ERROR";
            }
       }
       catch (NumberFormatException ex)
       {
           return "ERROR";
       }
       
    }   

    
}
