/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.mycompany._orienteering;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.Scanner;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author gian
 */
public class UDPServer implements Runnable
{
    private DatagramSocket socket;
    //Massimo numero di byte dei dati di una request
    // e di una response
    private final int MAX_LENGHT_DATA=1024; 
    private Integer numeroProtocollo=0;
    private Integer annoProtocollo=0;
    private String fileName="orienteering.csv";
    
    public UDPServer(int port) throws SocketException
    {
        //Istanzia un socket per il server attivo sulla 
        //porta "port"
        socket=new DatagramSocket(port);
        //Imposta un Timeout, quando il server sarà in ascolto,
        //dopo un secondo si verifica un'eccezione SocketTimeoutException
        socket.setSoTimeout(1000);
    }
    
    public void run()
    {
        byte[] requestBuffer=new byte[MAX_LENGHT_DATA];
        DatagramPacket request=new DatagramPacket(requestBuffer, requestBuffer.length);
        String requestString;
        String responseString;
        DatagramPacket response;
        byte[] responseBuffer;
        //ciclo:
        //riceve request, costruisce repoinse, 
        //invia la response
        while(!Thread.interrupted())
        {
            try 
            {
                //metto il socket in ascolto sulla sua porta
                //dopo un secondo si verificherà l'eccezione
                socket.receive(request);
                //leggo i dati contenuti nella request (è una stringa)
                requestString=new String(requestBuffer, "UTF-8");
                //costruisco la risposta
               // responseString=requestString.substring(0,request.getLength()); //sotto forma di stringa
               requestString=requestString.substring(0,request.getLength());
               
               responseString=analisiRequest(requestString);
                
               responseBuffer=responseString.getBytes(); //sotto forma di array di byte
               response=new DatagramPacket(responseBuffer, responseString.length(), request.getAddress(), request.getPort());
                //spedire response
                socket.send(response);
            } 
            catch (SocketTimeoutException ex) 
            {
                System.out.println("TIMEOUT");
            }
            catch (IOException ex) 
            {
                System.out.println("Errore di comunicazione");
            }
        }
        
        socket.close(); 
    }
    
    
    public static void main(String[] args) 
    {
         Scanner tastiera=new Scanner(System.in);
        try 
        {
            UDPServer UDPserver=new UDPServer(7);
            Thread server=new Thread(UDPserver);
            server.start();
            tastiera.nextLine();
            server.interrupt();
            server.join();
            System.out.println("Server interrotto");
        } 
        catch (SocketException ex) 
        {
            System.out.println("Impossibile istanziare il socket del server");
        } 
        catch (InterruptedException ex) 
        {
             System.out.println("Impossibile interrompere il server");
        }
    }

    private void leggiDaFile() throws IOException, FileException 
    {
        
        TextFile file=new TextFile(fileName, 'R');
        String rigaLetta;
        String[] elementiRiga;
        
        
        
        rigaLetta=file.fromFile();
        elementiRiga=rigaLetta.split("/");
        
        numeroProtocollo=Integer.parseInt(elementiRiga[1]);
        annoProtocollo=Integer.parseInt(elementiRiga[0]);
        if (annoProtocollo!=LocalDateTime.now().getYear())
            annoProtocollo=LocalDateTime.now().getYear();
        file.close();
    }
    
    private void scriviSuFile() throws IOException, FileException
    {
        TextFile file=new TextFile(fileName, 'W');
        file.toFile(Integer.toString(annoProtocollo)+"/"+Integer.toString(numeroProtocollo));
        file.close();
    }

    private String analisiRequest(String requestString) 
    {
       String[] elementiRequest=requestString.split(";");
       String codiceTappa=elementiRequest[0];
       String codiceConcorrente=elementiRequest[1];
       String rigaDaScrivere;
       
       //Verifico la prima parte del messaggio
       int numeroTappa, numeroConcorrente;
       
       if (elementiRequest.length!=2)
           return "CLIENT ERROR";
       if(!codiceTappa.substring(0,4).equals("DART"))
           return "CLIENT ERROR";
       try
       {
            //Verifico il numero di tappa (se è un numero)
            numeroTappa=Integer.parseInt(codiceTappa.substring(4,codiceTappa.length()));
            //verifico il numero del concorrente (se è un numero)
            numeroConcorrente=Integer.parseInt(codiceConcorrente);
       }
       catch(NumberFormatException ex)
       {
           return "CLIENT ERROR";
       }
              
      
       //Costrruisco la riga da memorizzare sul dile aggiungendo il timestamp di epoch
       LocalDateTime adesso=LocalDateTime.now();
       Long numeroSecondi=adesso.toEpochSecond(ZoneOffset.UTC);
       rigaDaScrivere=requestString+";"+Long.toString(numeroSecondi);
       
        TextFile file;
        try 
        {
            file = new TextFile(fileName, 'W',true);
            file.toFile(rigaDaScrivere);
            file.close();
        } 
        catch (IOException ex) 
        {
            return "SERVER ERROR";
        } 
        catch (FileException ex) 
        {
              return "SERVER ERROR";
        }
        
       return "OK";
       
    }
    
    
}
