/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package pkg2_numeroprotocollo;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.time.LocalDateTime;
import java.util.Scanner;
import java.util.logging.Level;
import java.util.logging.Logger;
import utilita.FileException;
import utilita.TextFile;

/**
 *
 * @author gian
 * Protocoolo che restituisce un numero di protocollo, il numero di protocollo ha la seguente forma
 * AAAA/numero  dove AAAA è l'anno corrente e numero è un numero progressivo che si incrmenta ogni volta
 * Protocollo:
 * se request= '?' --> response--> numero di protocollo
 * Esempi di numero di protocollo:
 *      2024/234
 *      2024/235
 *      2025/1
 *      2025/2
 */
public class UDPServer implements Runnable
{
    private DatagramSocket socket;
    //Massimo numero di byte dei dati di una request
    // e di una response
    private final int MAX_LENGHT_DATA=1024; 
    private int protocollo=0;
    private int anno;
    private String filename="protocollo.txt";
    
    public UDPServer(int port) throws SocketException, IOException, FileException
    {
        //Istanzia un socket per il server attivo sulla 
        //porta "port"
        socket=new DatagramSocket(port);
        //Imposta un Timeout, quando il server sarà in ascolto,
        //dopo un secondo si verifica un'eccezione SocketTimeoutException
        socket.setSoTimeout(1000);
        //Carico da file l'ultimo numero di protocollo inviato
        leggiProtocollo();
    }
    
    public void run()
    {
        byte[] requestBuffer=new byte[MAX_LENGHT_DATA];
        DatagramPacket request=new DatagramPacket(requestBuffer, requestBuffer.length);
        String requestString;
        String responseString;
        DatagramPacket response;
        byte[] responseBuffer;
        
        //ciclo:
        //riceve request, costruisce repoinse, 
        //invia la response
        while(!Thread.interrupted())
        {
            try 
            {
                //metto il socket in ascolto sulla sua porta
                //dopo un secondo si verificherà l'eccezione
                socket.receive(request);
                //leggo i dati contenuti nella request (è una stringa)
                requestString=new String(requestBuffer, "UTF-8");
                requestString=requestString.substring(0,request.getLength());
                //verifico se il messaggio ricevuto è corretto
                if (requestString.equals("?"))
                {
                     try 
                     {
                       //costruisco la risposta
                       responseString=aggiornaProtocollo(); //sotto forma di stringa
                     //  responseString=Integer.toString(LocalDateTime.now().getYear())+"/"+responseString;
                     } 
                     catch (FileException ex) 
                     {
                         responseString="Server Error";
                     }
                }
                  
                else
                    responseString="Error";
                responseBuffer=responseString.getBytes(); //sotto forma di array di byte
                response=new DatagramPacket(responseBuffer, responseBuffer.length, request.getAddress(), request.getPort());
                //spedire response
                socket.send(response);
            } 
            catch (SocketTimeoutException ex) 
            {
                System.out.println("TIMEOUT");
            }
            catch (IOException ex) 
            {
                System.out.println("Errore di comunicazione");
            }
        }
        
        socket.close(); 
    }
    
    private void leggiProtocollo() throws IOException, FileException
    {
        String rigaLetta;
        String[] valoriLetti;
        TextFile file=new TextFile(filename, 'R');
        rigaLetta=file.fromFile();
        file.close();
        valoriLetti=rigaLetta.split("/");
        protocollo=Integer.parseInt(valoriLetti[1]);    
        anno=Integer.parseInt(valoriLetti[0]); 
        if (anno!=LocalDateTime.now().getYear())
        {
            protocollo=0; 
            anno=LocalDateTime.now().getYear();
        }
                   
    }
    
    private String aggiornaProtocollo() throws IOException, FileException
    {
        String stringaProtocollo;
        protocollo++;
        stringaProtocollo=Integer.toString(anno)+"/"+Integer.toString(protocollo);
        TextFile file=new TextFile(filename, 'W');
        file.toFile(stringaProtocollo);
        file.close();
        return stringaProtocollo;
        
    }
    
    public static void main(String[] args) 
    {
         Scanner tastiera=new Scanner(System.in);
        try 
        {
            UDPServer UDPserver=new UDPServer(1200);
            Thread server=new Thread(UDPserver);
            server.start();
            tastiera.nextLine();
            server.interrupt();
            server.join();
            System.out.println("Server interrotto");
        } 
        catch (SocketException ex) 
        {
            System.out.println("Impossibile istanziare il socket del server");
        } 
        catch (InterruptedException ex) 
        {
             System.out.println("Impossibile interrompere il server");
        } 
        catch (IOException | FileException ex)
        {
            System.out.println("Errore lettura file");
        } 
   
    }
    
    
}
