/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.mycompany._calcolatrice_udp;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.util.Scanner;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author gian
 */
public class UDPServer implements Runnable
{
    private DatagramSocket socket;
    final int MAX_LENGTH_DATA=8192;
    
    public UDPServer(int port) throws SocketException
    {
        //istanzio il socket per leggere/scrivere dati sulla prota "port"
        socket=new DatagramSocket(port);
        //assegno un timeout di 1 s, dopo un secondo il socket si chiude e poi si rimetterà in ascolto
        socket.setSoTimeout(1000);
    }
    
    String calcolo(byte[] requestData) throws UnsupportedEncodingException
    {
        String requestMessage=new String(requestData,"UTF-8");
        String[] requestElements;
        double x,y,risultato;
        String operazione;
        
        
        requestElements=requestMessage.split(",");
        operazione=requestElements[0];
        x=Double.parseDouble(requestElements[1]);
        y=Double.parseDouble(requestElements[2]);
        
        switch (operazione) {
            case "ADD":
                risultato=x+y;
                break;
            case "SUB":
                risultato=x-y;
                break;
            case "MUL":
                risultato=x*y;
                break;
            case "DIV":
                if (y==0)
                    return "divisione per 0";
                else
                    risultato=x/y;
                break;
            default:
                return "ERROR";
        }
        
        return Double.toString(risultato);
    }
    
    public void run()
    {
        byte[] data=new byte[MAX_LENGTH_DATA];
        //istanzio un datagramma vuoto che ospiterà il datagarmma ricevuto
        DatagramPacket request=new DatagramPacket(data, data.length);
        //reference al datagramma di risposta
        DatagramPacket response;
        String responseMessage;
        while(!Thread.interrupted())
        {
            try 
            {
                //si mette in ascolto sulla prota "port" in attesa di un datagramma
                //dopo un secondo interromperà l'ascolto e poi si rimetterà in ascolto (Timeout)
                socket.receive(request);
                //effettuo il calcolo
                try
                {
                    responseMessage=calcolo(request.getData());
                }
                catch (UnsupportedEncodingException ex)
                {
                        responseMessage="Formato richiesta non corretto";
                }
                
                
                //se arriva la request, costruisco la response estraendo i dati dalla request
                response=new DatagramPacket(responseMessage.getBytes("UTF-8"), responseMessage.length(),request.getAddress(),request.getPort());
                //invio la response
                socket.send(response);
            } 
            
            catch (SocketTimeoutException ex) 
            {
                System.out.println("Timeout");
            }
            catch (IOException ex) 
            {
                System.out.println("Errore di comuniczione");
            }
        }
        socket.close();
        System.out.println("Server interrotto");
    }
    
    public static void main(String[] args) 
    {
        Scanner tastiera=new Scanner(System.in);
        try 
        {
            
            UDPServer echoServer=new UDPServer(7);
            Thread echoServerThread=new Thread(echoServer);
            echoServerThread.start();
            tastiera.nextLine();
            echoServerThread.interrupt();
        } 
        catch (SocketException ex) 
        {
            System.out.println("Impossibile aprire il socket del server");
        }
    }

}
