/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package Server;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.SocketTimeoutException;
import java.util.ArrayList;
import java.util.Scanner;
import java.util.logging.Level;
import java.util.logging.Logger;



/**
 *
 * @author gian
 */
public class UDPServer implements Runnable
{
  
    private DatagramSocket socket;
    private final int MAX_BYTE_REQUEST=8192;
    private final int MAX_BYTE_RESPONSE=8192;
    //private String fileName="messaggi.txt";
    private ArrayList<Utente> elencoUtenti;
    
    public UDPServer(int port) throws SocketException
    {
        socket=new DatagramSocket(port);
        socket.setSoTimeout(1000);
        elencoUtenti=new ArrayList<Utente>();
    }
    
    private String aggiungiUtente(String name,String ipAddress,int port)
    {
        Utente nuovUtente=new Utente(name, ipAddress, port);
        //boolean giaPresente=false;
        for(Utente u:elencoUtenti)
        {
            if (u.equals(nuovUtente))
            {
                return "ERROR";
            }
        }
       // if(!giaPresente)
        elencoUtenti.add(nuovUtente);
        return "OK";   
    }
    
    //Verifico la presenza dell'utente dalla coppia IP/Porta
    private Utente controllaPresenzaUtente(String ip, int port)
    {
        for(Utente u:elencoUtenti)
        {
            if (u.getIpAddress().equals(ip) && u.getPort()==port)
            {
                return u;
            }
        }
        return null;
    }
    
    public void run()
    {
        byte[] requestBuffer=new byte[MAX_BYTE_REQUEST];
        //byte[] responseBuffer=new byte[MAX_BYTE_RESPONSE];
        DatagramPacket requestPacket;
        DatagramPacket responsePacket;
        String requestMessage;
        String responseMessage="ERROR";
        String ipClient;
        int portClient;
        String clientName;
        
        while(!Thread.interrupted())
        {
            
            try 
            {
                requestPacket=new DatagramPacket(requestBuffer, MAX_BYTE_REQUEST);
                socket.receive(requestPacket);
                //Estraggo dalla request tutte le informazioni
                ipClient=requestPacket.getAddress().toString();
                ipClient=ipClient.substring(1,ipClient.length()); //mi serve per togliere lo slash iniziale
                portClient=requestPacket.getPort();
                requestMessage=new String(requestPacket.getData(),"UTF-8");
                requestMessage=requestMessage.substring(0, requestPacket.getLength());
                //verifico se il messggio è una richiesta di iscrizione
                //in tal caso aggiungo l'utente se non è già presente nell'array list
                //il metodo aggiungiUtente restituisce sempre OK
                if (requestMessage.length()>=3 && requestMessage.substring(0,3).equals("ADD"))
                {
                  responseMessage=aggiungiUtente(requestMessage.substring(4,requestMessage.length()),ipClient,portClient);
                  responsePacket=new DatagramPacket(responseMessage.getBytes(),responseMessage.length(),requestPacket.getAddress(), requestPacket.getPort());
                  socket.send(responsePacket);
                  System.out.println("Risposta inviata al client "+ipClient+"/"+portClient);
                
                }
                else //Il messaggio non è una richiesta di iscrizione
                {
                    //verifico se l'utente è iscritto
                    Utente utenteClient=controllaPresenzaUtente(ipClient,portClient);
                    if(utenteClient!=null)
                    {
                        //messaggio da invire a tutti
                        responseMessage=utenteClient.getName()+" --> "+new String(requestPacket.getData(),"UTF-8");
                        responseMessage=responseMessage.substring(0,utenteClient.getName().length()+5+requestPacket.getLength());
                        //Se l'utente è iscritto, il suo messaggio viene inviato a tutti gli utenti
                        for (Utente u:elencoUtenti)
                        { 
                            responsePacket=new DatagramPacket(responseMessage.getBytes(),responseMessage.length(),InetAddress.getByName(u.getIpAddress()),u.getPort() );
                            socket.send(responsePacket);
                            System.out.println("Risposta inviata al client "+ipClient+"/"+portClient);
                        }
                    }
                    else
                    {
                        //L'utente non è iscritto
                        //il responseMessage vale ERROR ed è inviato solo all'utente che ha inviato la request
                        responsePacket=new DatagramPacket(responseMessage.getBytes(),responseMessage.length(),requestPacket.getAddress(),requestPacket.getPort());
                        socket.send(responsePacket);
                        System.out.println("Risposta inviata al client "+ipClient+"/"+portClient);
                    }    
                }
                
                //costruisco il messaggio di risposta
                //responseMessage="bla bla";            
            } 
            catch (SocketTimeoutException ex) 
            {
                System.out.println("Timeout");
            }
            catch (IOException ex) 
            {
                System.out.println("Errore di comunicazione");
            }
        }
        socket.close();
    }
    
    public static void main(String[] args) 
    {
        Scanner tastiera=new Scanner(System.in);
        
        try 
        {
            UDPServer chatServer=new UDPServer(1212);
            Thread chatServerThread=new Thread(chatServer);
            chatServerThread.start();
            tastiera.nextLine();
            chatServerThread.interrupt();
            chatServerThread.join();
            System.out.println("Server disconnesso");
           
        } 
        catch (SocketException ex) 
        {
            System.out.println("Impossibile aprire il socket");
        } 
        catch (InterruptedException ex) 
        {
            System.out.println("Impossibile interrompere il server");
        }
    }

}
